from dotenv import load_dotenv
import os


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]


class Config(metaclass=Singleton):
    # _KEYS = [
    #     "PUSHOVER_USER",
    #     "PUSHOVER_API_TOKEN",
    # ]

    def __init__(self):
        import alertist.targets

        load_dotenv()
        for t in alertist.targets.__all__:
            klass = getattr(alertist.targets, t)
            for k in klass.CONFIG_KEYS:
                setattr(self, k, os.environ.get(k))



from abc import ABC
from ..conf import Config


class ABCTarget(ABC):
    def __init__(self):
        self.validate_configuration()

    def validate_configuration(self):
        for cfg in self.CONFIG_KEYS:
            if getattr(Config(), cfg, None) is None:
                raise ValueError(f"{cfg} is not set")

    def alert(self, message: str, **kwargs):
        raise NotImplementedError

    async def aalert(self, message: str, **kwargs):
        raise NotImplementedError



from .conf import Config
from typing import Optional, List
from .targets.abstract import ABCTarget
import os


class Alertist:
    ALERTS = []

    def __init__(
        self,
        alert: Optional[ABCTarget] = None,
        # alerts: Optional[List[ABCTarget]] = None,
    ) -> None:
        Config()
        if alert is not None:
            self.ALERTS = [alert]
        # TODO support for multiple
        # elif alerts is not None:
        #     self.ALERTS = alerts
        # else:
        #     pass
        # TODO dynamic alerts here, this is a placeholder
        # keys = os.getenv("ALERTS", "").split(",")
        # self.ALERTS = keys

        if len(self.ALERTS) == 0:
            raise ValueError("Configure an alert target")

    def alert(self, message: str, **kwargs):
        for a in self.ALERTS:
            a().alert(message, **kwargs)

    async def aalert(self, message: str, **kwargs):
        for a in self.ALERTS:
            await a().aalert(message, **kwargs)



from ..conf import Config
import httpx
from .abstract import ABCTarget
from typing import Optional


class Pushover(ABCTarget):
    """A class for alerting via Pushover (https://pushover.net/)"""

    _url_ = "https://api.pushover.net:443/1/messages.json"

    CONFIG_KEYS = ["PUSHOVER_API_TOKEN", "PUSHOVER_USER"]

    def alert(self, message: str):
        data = {
            "token": Config().PUSHOVER_API_TOKEN,
            "user": Config().PUSHOVER_USER,
            "message": message,
        }
        r = httpx.post(self._url_, data)

    async def aalert(self, message: str):
        data = {
            "token": Config().PUSHOVER_API_TOKEN,
            "user": Config().PUSHOVER_USER,
            "message": message,
        }
        await httpx.AsyncClient().post(self._url_, data=data)
